import torch
import torch.distributed.autograd as dist_autograd
import torch.distributed.rpc as rpc
import torch.multiprocessing as mp
import torch.nn as nn
import torch.nn.functional as F
from . import Net
from torch import optim
from torch.distributed.optim import DistributedOptimizer
from torchvision import datasets,transforms
def call_method(method,rref,*args,**kwargs):
    return method(rref.local_value(),*args,**kwargs)


def remote_method(method,rref,*args,**kwargs):
    args=[method,rref]+list(args)
    return rpc.rpc_sync(rref.owner(),call_method,args=args,kwargs=kwargs)

class ParameterServer(nn.Module):
    def __init__(self,num_gpus):
        super().__init__()
        self.model=Net(num_gpus)
        self.input_device = torch.device("cuda:0" if torch.cuda.is_available() and num_gpus > 0 else "cpu")
    
    def forward(self, inp):
        inp = inp.to(self.input_device)
        out = self.model(inp)
        # This output is forwarded over RPC, which as of 1.5.0 only accepts CPU tensors.
        # Tensors must be moved in and out of GPU memory due to this.
        out = out.to("cpu")
        return out
    def get_dist_gradients(self,cid):
        grads=dist_autograd.get_gradients(cid)
        cpu_grads={}
        for k,v in grads.items():
            k_cpu,v_cpu=k.to("cpu"),v.to("cpu")
            cpu_grads[k_cpu]=v_cpu